#!/usr/bin/perl -w

use strict;

# extract.pl
# Extract training examples from CMU Seminars
# Fraser

my @tokens = ();
my @classes = ();

while (<STDIN>) {
    chomp;
    
    # make sure we tokenize before start tags and after end tags
    s/<stime>/ <stime>/g;
    s/<\/stime>/<\/stime> /g;

    # normalize AM/PM
    s/a\.m\.?/ AM /ig;
    s/p\.m\.?/ PM /ig;

    # separate commas, dashes
    s/,/ , /g;
    s/-/ - /g;

    # separate parentheses
    s/\(/ ( /g;
    s/\)/ ) /g;

    # only separate period followed by space or end of line
    s/\.( |$)/ . /g;

    my @spacesep = split(/\s+/, $_);

    for (my $i = 0; $i < @spacesep; $i++) {
	if ($spacesep[$i] =~ /^<stime>/) {
	    push(@classes, 1);
	}
	else {
	    push(@classes, 0);
	}
	$spacesep[$i] =~ s/<.*?>//g;

	if ($spacesep[$i] eq /^\s+$/) {
	    push(@tokens, "NULL");
	}
	else {
	    push(@tokens, $spacesep[$i]);
	}
    }
}

# we have read in the whole file, we have @tokens and @classes
for (my $i = 0; $i < @tokens; $i++) {
    # Print 4 features and label

    # Feature 1 is word to left
    if ($i == 0) {
	print "START_OF_FILE ";
    }
    else {
	print $tokens[$i-1], " ";
    }
    
    # Feature 2 is current token
    print $tokens[$i], " ";

    # Feature 3 is next token
    if ($i == @tokens-1) {
	print "END_OF_FILE ";
    }
    else {
	print $tokens[$i+1], " ";
    }

    # Feature 4 is token+2
    if ($i >= @tokens-2) {
	print "END_OF_FILE ";
    }
    else {
	print $tokens[$i+2], " ";
    }

    # Feature 5 is token+3
    if ($i >= @tokens-3) {
	print "END_OF_FILE ";
    }
    else {
	print $tokens[$i+3], " ";
    }

    # Feature 6 is whether current token could look like a time (digit colon digit)
    if ($tokens[$i] =~ /\d:\d/) {
	print "DCD ";
    }
    else {
	print "NO ";
    }

    # Feature 7 is next token
    if ($i == @tokens-1) {
	print "NO_TOKEN ";
    }
    elsif ($tokens[$i+1] =~ /\d:\d/) {
	print "DCD ";
    }
    else {
	print "NO ";
    }

    # Feature 8 is token+2
    if ($i >= @tokens-2) {
	print "NO_TOKEN ";
    }
    elsif ($tokens[$i+2] =~ /\d:\d/) {
	print "DCD ";
    }
    else {
	print "NO ";
    }

    # Feature 9 is token+3
    if ($i >= @tokens-3) {
	print "END_OF_FILE ";
    }
    elsif ($tokens[$i+3] =~ /\d:\d/) {
	print "DCD ";
    }
    else {
	print "NO ";
    }

    print $classes[$i], "\n\n";
}
